/*
Portable ZX-Spectrum emulator.
Copyright (C) 2001-2020 SMT, Dexus, Alone Coder, deathsoft, djdron, scor

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package app.usp.ctl;

import android.content.Context;
import android.content.res.Configuration;
import android.widget.ImageView;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import app.usp.Emulator;
import app.usp.Preferences;
import app.usp.R;

public class Control extends ImageView
{
    private Bitmap keyboard;
    private Bitmap joystick;
    private boolean keyboard_active = false;

	public Control(Context context)
	{
		super(context);

		int width = 0;
		if(getResources().getConfiguration().orientation == Configuration.ORIENTATION_PORTRAIT)
			width = context.getResources().getDisplayMetrics().widthPixels;
		else
			width = context.getResources().getDisplayMetrics().heightPixels;
		BitmapFactory.Options options = new BitmapFactory.Options();
		options.inScaled = false;
		if(width > 480)
		{
			keyboard = BitmapFactory.decodeResource(getResources(), R.drawable.keyboard_hq, options);
			joystick = BitmapFactory.decodeResource(getResources(), R.drawable.joystick_hq, options);
		}
		else
		{
			keyboard = BitmapFactory.decodeResource(getResources(), R.drawable.keyboard, options);
			joystick = BitmapFactory.decodeResource(getResources(), R.drawable.joystick, options);
		}
		keyboard_active = Emulator.the.GetOptionBool(Preferences.use_keyboard_id);
		setAdjustViewBounds(true);
		setScaleType(ScaleType.FIT_XY);
		setImageBitmap(keyboard_active ? keyboard : joystick);
		setOnTouchListener(new ControlTouch() {
			public void OnTouch(float x, float y, boolean down, int pid)
			{
				Emulator.the.OnTouch(keyboard_active, x/getWidth(), y/getHeight(), down, pid);
			}}
		);
	}
	@Override
	protected void onMeasure(int w, int h)
	{
		if(getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE)
			setMeasuredDimension(0, 0);
		else
			super.onMeasure(w, h);
	}
	public void OnToggle()
	{
		keyboard_active = !keyboard_active;
		Emulator.the.SetOptionBool(Preferences.use_keyboard_id, keyboard_active);
		setImageBitmap(keyboard_active ? keyboard : joystick);
	}

	public static void UpdateJoystickKeys(final boolean left, final boolean right, final boolean up, final boolean down)
	{
		// release keys first
		if(!left)
			Emulator.the.OnKey('l', false, false, false);
		if(!right)
			Emulator.the.OnKey('r', false, false, false);
		if(!up)
			Emulator.the.OnKey('u', false, false, false);
		if(!down)
			Emulator.the.OnKey('d', false, false, false);

		// then press keys
		if(left)
			Emulator.the.OnKey('l', true, false, false);
		if(right)
			Emulator.the.OnKey('r', true, false, false);
		if(up)
			Emulator.the.OnKey('u', true, false, false);
		if(down)
			Emulator.the.OnKey('d', true, false, false);
	}
}
